#!/bin/bash
# fetchmail.sh - Andreas Rehm ITsolution
# klaus deiss it-userdesk.de k.deiss@it-userdesk.de
# Stand 0.6
#
# Diese Script muss mit einer bash / cygwin oder auf einem Unix Rechner gestartet werden
# Es werden automatisch pro Tag 3 Logfiles angelegt.
# <DATUM>-fetchmail = logs entire communication
# <DATUM>-short = logs essential part of communication (from/to/etc)
# ERR_<DATUM> = logs errors
#
# Es wird nur einmal fetchmail gestartet, wenn es bereits luft wird abgebrochen
# Archivierung alter Logfiles nach 7 Tagen mit bzip2 und automatische Lschung nach einem Jahr
#
# needs fetchmail / bzip2 / egrep
#
#
##############################################################################
# History
# V0.1 - Erste Version
# V0.2 - Fehlerbereinigung Lschen alter Logdateien
# V0.3 - Deiss: Neuer Logfiletyp nur bei erfolgtem SMTP Transfer
# V0.4 - Behandlung von SMTP Fehlern /size exceeded etc.
# V0.5 - regelmaessiger tar/bzip des /log/tmp ordners
# V0.6 - 2.4.2012 erfasst auch Fehler die im Resultcode von fetchmail nicht auftauchen !
##############################################################################
# Start Skript unter Windows
# @echo off
# C:
# chdir C:\cygwin\bin
# bash --login -i /fetchmail.sh
# exit

# Abbruch, wenn fetchmail noch luft
# ps -a | grep fetchmail && exit;

# Start
echo "Starte E-Mail Abholung"

# Neue Logdatei ja / nein
NEWLOGFILE=0

# Log-Datei erstellen - LOGDATE ist das aktuelle Datum
LOGDATE=`date +'%Y-%m-%d'`
# Logdir ist der Ordner mit den Logdateien - dieser wird ggf. erstellt
LOGDIR='/log'
# Logfile ist die Logdatei, die sich aus den vorhergehenden Daten zusammensetzt
LOGFILE="${LOGDIR}/${LOGDATE}-fetchmail.log"

#mod by deiss
LOGDIR2='/log/tmp'
LOGDATETMP=`date +'%Y-%m-%d-%H-%M-%S'`-$$
LOGFILEPOP="${LOGDIR2}/${LOGDATETMP}.pop"
LOGFILETMP="${LOGDIR2}/${LOGDATETMP}.tmp"

#moegliche Fehlereintraege
SMTPERR="SMTP error| connect to .* failed"
OTHERERR="Query status="

#sicherstellen das ein logfile fuer fetchmail existiert !
date > ${LOGFILETMP}
SHORTLOGFILE="${LOGDIR}/${LOGDATE}-short.log"
ERRORLOG="${LOGDIR}/ERR_`date +'%Y-%m-%d'`.log"


# diese funktion analysiert ein einzelnes temp file und holt 
# die smpt kommunikation heraus
# set -x

function ana_tmp
{
file=$1
let RTC=0
if [ -f $file ] ; then
    errline=`cat "${file}" | egrep "$SMTPERR" > /dev/nul`
    if [ $? -eq 0 ] ; then
	    # wir haben einen fehler in der smpt kommunikation !!
	    # definition der fehler $SMTPERR wird mit egrep rausgefischt
	    echo "`date` fetchmail SMTP Err - Error follows.">> ${ERRORLOG}
	    echo "$errline">> ${ERRORLOG}
	    echo "">> ${ERRORLOG}
	    echo "------------------------------------------">> ${ERRORLOG}
	    cat ${file} >> ${ERRORLOG}
	    echo "">> ${ERRORLOG}
	    echo "">> ${ERRORLOG}
	    let RTC=1
    else
    	    date +'%Y-%m-%d %H:%M' >> "${SHORTLOGFILE}"
	    echo "Details: ${LOGFILETMP}" >> "${SHORTLOGFILE}"
	    cat "${file}" | grep "message for" >> "${SHORTLOGFILE}"
	    cat "${file}" | grep "messages for" >> "${SHORTLOGFILE}"
	    cat "${file}" | grep "MAIL FROM" -A 1 >> "${SHORTLOGFILE}"
	    cat "${file}" | grep "RCPT TO" -A 1 >> "${SHORTLOGFILE}"
	    cat "${file}" | grep "EOM" -A 1 >> "${SHORTLOGFILE}"
	    echo "" >> "${SHORTLOGFILE}"    
	    let RTC=0
    fi
    rm -f $file
else
    echo "`date` No access to file $file !" >> $ERRORLOG
    let RTC=2
fi
return $RTC
}

function exit_fetch
{
# Ende protokollieren
DATEEND=`date +'%d.%m.%Y %H:%M:%S'`
echo -e "ENDE der E-Mailabholung am ${DATEEND}\n" >> ${LOGFILE}

# Logrotation wenn eine neue Datei angelegt wurde
if [ "${NEWLOGFILE}" == "1" ];
then
	date +'%Y-%m-%d %H:%M' >> "${SHORTLOGFILE}"
	echo -e "Logrotation ${DATEEND}\n" >> ${SHORTLOGFILE}
	
	# Dateien die lter als 7 Tage sind werden komprimiert
	find ${LOGDIR} -name \*.log -ctime 7 -exec bzip2 {} \;
	
	# Dateien die lter als 365 Tage sind werden gelscht
	find ${LOGDIR} -name \*.log.bz2 -ctime 365 -exec rm {} \;

	# LOGFILETMP - hier liegen nur die erfolgten Abholungen als einzeldatei
	# mit tar zusammenbauen und dann zippen
	BFN="$LOGDIR2/$(date +%Y%m%d)"
	find ${LOGDIR2} -type f -name *.pop -print | xargs tar -cf $BFN.tar
	bzip2 $BFN.tar
	find ${LOGDIR2} -type f -name *.pop -exec rm {} \;
	
	echo -e "Logrotation beendet.\n" >> ${SHORTLOGFILE}
	date +'%Y-%m-%d %H:%M' >> "${SHORTLOGFILE}"
fi
}

# Logdir erstellen wenn noetig
if [ ! -d ${LOGDIR} ]
then
	echo "Erstelle Logdir = ${LOGDIR}"
	mkdir -p ${LOGDIR}
fi

#deiss mod unterhalb von log ein temp ordner anlegen
if [ ! -d ${LOGDIR2} ]
then
	echo "Erstelle Logdir = ${LOGDIR2}"
	mkdir -p ${LOGDIR2}
fi

# Logfile anlegen, ggfs Logrotation einleiten
if [ ! -f ${LOGFILE} ]
then
	echo "Logdatei ${LOGFILE} erstellen..."
	touch ${LOGFILE}
	NEWLOGFILE=1
fi


# Anfang protokollieren
DATE=`date +'%d.%m.%Y %H:%M:%S'`
echo "START der E-Mailabholung am ${DATE}" >> ${LOGFILE}

# E-Mail abholen
# wenn fetchmail noch laeuft wird abgebrochen
FETCH_PID=`ps -e | grep fetchmail`
if [ $? -gt 0 ] ; then
    fetchmail -v -f /fetchmailrc -L ${LOGFILETMP}
    RST=$?
else
    #fetchmail laeuft noch / Abbruch   
    echo "`date` fetchmail already running !!!" >> ${ERRORLOG}
    echo $FETCH_PID>> ${ERRORLOG}
    echo "">> ${ERRORLOG}
    echo "">> ${ERRORLOG}
    exit_fetch
    rm -f ${LOGFILETMP}
    exit
fi


#wenn der rstcode groesse 1 ist haben wir einen Fehler
if [ $RST -gt 1 ] ; then
    echo "`date` fetchmail resultcode $RST - Error follows.">> ${ERRORLOG}
    cat ${LOGFILETMP} >> ${ERRORLOG}
    echo "">> ${ERRORLOG}
    echo "">> ${ERRORLOG}
    echo "">> ${ERRORLOG}
fi

# deiss erzeugen eines gut lesbaren short logs
if [ -f "${LOGFILETMP}" ] ; then
	cat "${LOGFILETMP}" | grep "SMTP" > /dev/null
        if [ $? -gt 0 ] ; then
	    #wir haben keinen smtp traffic in der datei
	    if [ $RST -eq 0 ] ; then
		#solte das eintreffen haben wir keine gueltige ausgabe von fetchmail !
		echo "`date` fetchmail results no error but no SMTP transfer - Error follows.">> ${ERRORLOG}
	        cat ${LOGFILETMP} >> ${ERRORLOG}
		echo "">> ${ERRORLOG}
		echo "">> ${ERRORLOG}
	    else
		#hiermit werden Fehler abgefangen die Fetchmail ohne Resultcode zurückgibt
		#z.B ungueltige zertifikate
		#cat "${LOGFILETMP}" > /debugmsg.txt
		cat "${LOGFILETMP}" | egrep "$OTHERERR" > /dev/null
    		if [ $? -eq 0 ] ; then
		    echo "There was an error while retrieving mail!" >> ${LOGFILE}
		    cat "${LOGFILETMP}" | egrep "$OTHERERR" >> ${LOGFILE}
		    cat "${LOGFILETMP}" | egrep "$OTHERERR" >> ${ERRORLOG}
		    cat ${LOGFILETMP} >> ${ERRORLOG}
    		    echo "">> ${ERRORLOG}
    		    echo "">> ${ERRORLOG}
		fi
		
		if [ $RST -gt 1 ] ; then
		    echo "There was an error while retrieving mail! - see Resultcode $RST" >> ${LOGFILE}
		    echo "There was an error while retrieving mail! - see Resultcode $RST" >> ${ERRORLOG}
	    	    cat ${LOGFILETMP} >> ${ERRORLOG}
    		    echo "">> ${ERRORLOG}
    		    echo "">> ${ERRORLOG}

		fi
		
		echo "Nothing to do / result code was $RST"
		echo "Nothing to do / result code was $RST">> ${LOGFILE}
	    fi	
	else
	    #wir haben SMTP Traffic in der Datei, also analysieren
	    #date +'%Y-%m-%d %H:%M'
	    cat ${LOGFILETMP} >> ${LOGFILE}
	    echo "Fetchmail Resultcode was: $RST" >> ${LOGFILE}
	    echo "" >> ${LOGFILE}
    	    cp ${LOGFILETMP} ${LOGFILEPOP}
	    let ctr=0
	    #fn ist die datei in der wir die gesuchten zeilen sammeln
    	    fn="${LOGDIR2}/`date +'%s'`-$ctr.tmp"
	    date > "$fn"
	    while read olin
	    do
		#wenn wir MAIL FROM finden wird neue temp datei angelegt
		echo "$olin" | grep "MAIL FROM" > /dev/null
		if [ $? -eq 0 ] ; then
		    if [ $ctr -gt 0 ] ; then 
			# ein SMTP block faengt an - wir schicken ihn zur analyse!
			ana_tmp $fn
			ARST=$?
			let ctr=$ctr+1
    			fn="${LOGDIR2}/`date +'%s'`-$ctr.tmp"
		        date > $fn
			echo "$olin" >> "$fn"
		    else
			#echo "skipping $ctr"
			#dies gilt immer für die erste datei die fuer das protok. keine sinnvollen daten entaeält
			rm -f $fn
			let ctr=$ctr+1
    			fn="${LOGDIR2}/`date +'%s'`-$ctr.tmp"
			date > $fn
			echo "$olin" >> "$fn"
		    fi
		else
		    echo "$olin" >> "$fn"
		fi 
	    done < "${LOGFILETMP}"
	    #die letzte Datei muss mit beruecksichtigt werden !
    	    ana_tmp $fn
	fi
else
	echo "`date` fetchmail results no error but no LOGFILE ${LOGFILETMP}">> ${ERRORLOG}
	echo "">> ${ERRORLOG}
	echo "">> ${ERRORLOG}
fi

rm -f "${LOGFILETMP}"
#rm -f "${LOGDIR2}/*.tmp"
exit_fetch
